<?php
/**
 * Joomla Database Tool Converter (Standalone)
 *
 * @copyright   Copyright (c)2019-2023 Stergios Zgouletas / Web Expert Services Ltd
 * @license     GNU GPL v2 or - at your option - any later version
 * @package     jdbconvertertool
 */

@ini_set('max_execution_time',0);
ignore_user_abort(true);

define( '_JEXEC', 1 );
define( 'DS', DIRECTORY_SEPARATOR );
@header('Content-Type: text/html; charset=utf-8');
@header('Cache-Control: private');
@header('Pragma: no-cache');

define('JPATH_BASE',dirname(__FILE__));
require_once( JPATH_BASE .DS.'includes'.DS.'defines.php' );

require_once( JPATH_BASE .DS.'includes'.DS.'framework.php' );

if(version_compare(JVERSION,'4.0.0','ge'))
{
	echo 'This tool is for Joomla 3.x versions';
	exit;
}
$app =JFactory::getApplication('site');

$convertEngine=$app->input->getInt('changetblengineinno',0)+$app->input->getInt('changetblenginemyisam',0);
$changeDatabaseCollation=$app->input->getInt('changedbcol',0);
$changeTableCollation=$app->input->getInt('changetblcol',0);

if($convertEngine==1 || $changeDatabaseCollation || $changeTableCollation)
{
	$config=JFactory::getConfig();
	$db = JFactory::getDbo();

	$dbName=$config->get('db');
	$messages=array();
	if($convertEngine==1)
	{
		$toInno=$app->input->getInt('changetblengineinno',0);
		$enigne=$toInno?'InnoDB':'MyISAM';
		$opengine=!$toInno?'InnoDB':'MyISAM';
		
		title('Database Converter '.$opengine.' to '.$enigne);
		$db->setQuery('SELECT table_name FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA ='.$db->quote($dbName).' AND `engine`='.$db->quote($opengine));
		$tables=$db->loadColumn();
		if(count($tables))
		{	
			$cc=0;
			foreach($tables as $table)
			{
				if(executeSQL('ALTER TABLE '.$db->quoteName($table).' ENGINE='.$enigne.';'))
				{
					msg('Table '.$table.' converted to '.$enigne,'success');
					$cc++;
				}
				else
				{
					msg('Table '.$table.' convert to '.$enigne.' failed','danger');
				}
			}
			msg('Total '.$opengine.' Tables'.count($tables).' Total Converted to '.$enigne.':'.$cc);
		}
		else
		{
			msg('All tables are '.$enigne.', Well done!','success');
		}
	}

	//Change Table Collation
	if($changeTableCollation)
	{
		title('Table Collation Convert');
		$db->setQuery('SHOW TABLES');
		$tables=$db->loadColumn();
		$break=false;
		foreach($tables as $table)
		{
			$db->setQuery('SELECT TABLE_COLLATION FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA ='.$db->quote($dbName).' AND TABLE_NAME ='.$db->quote($table));
			$collation=$db->loadResult();
			$changeCols=true;
			if($collation!='utf8mb4_general_ci')
			{
				if(!executeSQL('ALTER TABLE '.$db->quoteName($table).' CONVERT TO CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci;'))
				{
					$changeCols=false;
				}
				else
				{
					msg('Table '.$table.' convert to UFT8MB4','success');	
				}
			}
			
			if($changeCols && !$break)
			{								
				$db->setQuery('SHOW COLUMNS FROM '.$db->quoteName($table));
				$columns=$db->loadAssocList('Field');
				foreach($columns as $column)
				{	
					if(strpos($column['Type'],'char')!==false || strpos($column['Type'],'text')!==false)
					{
						$db->setQuery('SELECT character_set_name FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA ='.$db->quote($dbName).' AND TABLE_NAME ='.$db->quote($table).' AND COLUMN_NAME ='.$db->quote($column['Field']));
						$collation=$db->loadResult();	
						if($collation=='utf8mb4') continue;
						if(executeSQL('ALTER TABLE '.$db->quoteName($table).' MODIFY '.$db->quoteName($column['Field']).' '.$db->escape($column['Type']).' CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci;',$break))
						{
							msg('   > Column '.$column['Field'].' converted to UFT8MB4','success');
						}
						if($break) break;
					}
				}
			}
			else
			{
				msg('Table '.$table.' convert to UFT8MB4 failed','danger');
			}
			if($break) break;
		}
	}

	//Change Database Collation
	if($changeDatabaseCollation)
	{
		title('Database Collation Convert');
		$db->setQuery('SELECT default_character_set_name, default_collation_name FROM information_schema.schemata WHERE schema_name = '.$db->quote($dbName));
		$dbInfo=$db->loadObject();
		if($dbInfo->default_character_set_name=='utf8mb4' && $dbInfo->default_collation_name=='utf8mb4_general_ci')
		{
			msg('Database '.$dbName.' has collation UFT8MB4','success');
		}
		else
		{
			if(executeSQL('ALTER DATABASE '.$db->quoteName($dbName).' CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci;'))
			{
				msg('Database '.$dbName.' collation changed to UFT8MB4 from '.$dbInfo->default_character_set_name,'success');
			}
			else
			{
				msg('Database '.$dbName.' failed to change the collation','success');
			}
		}
	}
	$messages[]='<p><a href="'.$_SERVER['PHP_SELF'].'" class="btn btn-success">Go back and start over</a></p>';
	msg('IMPORTANT: DO NOT FORGET TO REMOVE THIS FILE FROM YOUR WEBSERVER!','danger');
}
else
{
	msg('<b>IMPORTANT: PLEASE BACK UP YOUR DATABASE FIRST!</b>','danger');
	$messages[]='<div>';
	$messages[]='<form method="POST" action="'.$_SERVER['PHP_SELF'].'?doprocess=1">';
	$messages[]='<label><input type="checkbox" value="1" name="changetblengineinno"> Convert Tables Engine from MyISAM to InnoDB</label><br/>';
	$messages[]='<label><input type="checkbox" value="1" name="changetblenginemyisam"> Convert Tables Engine from InnoDB to MyISAM</label><br/>';
	$messages[]='<label><input type="checkbox" value="1" name="changetblcol"> Change Tables Collation to utf8mb4_general_ci</label><br/>';
	$messages[]='<label><input type="checkbox" value="1" name="changedbcol"> Change Database Collation to utf8mb4_general_ci</label><br/>';
	$messages[]='<input type="submit" value="Start" class="btn btn-lg btn-success">';
	$messages[]='<p class="small">If your database is too large, select one option each time</p>';
	$messages[]='</form>';
	$messages[]='</div>';
}

function msg($msg,$type='info')
{
	global $messages;
	$messages[]='<div class="alert alert-'.$type.'">'.$msg.'</div>';
}
function title($msg,$size='h2')
{
	global $messages;
	$messages[]='<'.$size.' class="header">'.$msg.'</'.$size.'>';
}

function executeSQL($query,&$break=false)
{
	global $db;
	set_time_limit(0);
	try
	{
		$db->setQuery($query);
		return $db->execute();
	}
	catch(Exception $e)
	{
		msg($e->getMessage(),'error');
		$break=true;
		return false;
	}
}
?>
<!doctype html>
<html>
	<head>
		<title>Joomla Database Converter Tool</title>
		<link href="https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-Vkoo8x4CGsO3+Hhxv8T/Q5PaXtkKtu6ug5TOeNV6gBiFeWPGFN9MuhOf23Q9Ifjh" crossorigin="anonymous">
	</head>
	<body>
		<div class="container">
		<div class="row">
			<div class="col-sm-12">
				<h1 class="text-center">Joomla Database Converter Tool <span class="badge badge-info">v1.0.1</span></h1>
				<hr/>
				<?php echo implode("\n",$messages);?>
				<hr/>
				<div class="alert alert-info text-center">
					<p>Joomla Database Converter Tool v1.0.1</p>
					<p>Copyright (c) 2019-<?php echo date('Y');?> Web Expert Services Ltd / Stergios Zgouletas</p>
					<p><a href="https://www.web-expert.gr" target="_blank">Web-expert.gr</a></p>
					<hr/>
					<p>Joomla Database Converter Tool Software, distributed under the terms of the GNU General<br/>
					Public License version 3 or, at your option, any later version.<br/>
					This program comes with ABSOLUTELY NO WARRANTY as per sections 15 & 16 of the <a target="_blank" href="http://www.gnu.org/licenses/gpl-3.0.html">license</a>.</p>
					<form action="https://www.paypal.com/cgi-bin/webscr" method="post" target="_top">
					<input type="hidden" name="cmd" value="_s-xclick" />
					<input type="hidden" name="hosted_button_id" value="GMWH4MJRYWELS" />
					<input type="image" src="https://www.paypalobjects.com/en_US/i/btn/btn_donateCC_LG.gif" border="0" name="submit" title="PayPal - The safer, easier way to pay online!" alt="Donate with PayPal button" />
					<img alt="" border="0" src="https://www.paypal.com/en_GR/i/scr/pixel.gif" width="1" height="1" />
					</form>
				</div>
			</div>
		</div>
	</body>
</html>
<?php
$app->close();